from typing import *
import royalnet.constellation.api as rca
import royalnet.utils as ru
from ..tables import *


class ApiDiarioPagesStar(rca.ApiStar):
    path = "/api/diario/pages/v1"

    parameters = {
        "get": {
            "page": "The diario page you want to get. Can be negative to get the entries in reverse order."
        }
    }

    tags = ["diario"]

    @rca.magic
    async def get(self, data: rca.ApiData) -> ru.JSON:
        """Get a diario page made of up to 500 diario entries."""
        page_str = data["page"]
        try:
            page = int(page_str)
        except ValueError:
            raise rca.InvalidParameterError("'page' is not a valid int.")
        if page < 0:
            page = -page-1
            entries: List[Diario] = await ru.asyncify(
                data.session
                    .query(self.alchemy.get(Diario))
                    .order_by(self.alchemy.get(Diario).diario_id.desc()).limit(500)
                    .offset(page * 500)
                    .all
            )
        else:
            entries: List[Diario] = await ru.asyncify(
                data.session
                    .query(self.alchemy.get(Diario))
                    .order_by(self.alchemy.get(Diario).diario_id)
                    .limit(500)
                    .offset(page * 500)
                    .all
            )
        response = [entry.json() for entry in entries]
        return response



from sqlalchemy import *
from sqlalchemy.orm import relationship, composite
from sqlalchemy.ext.declarative import declared_attr
from ..types import LeagueRank, LeagueTier, LeagueLeague, Updatable


class LeagueOfLegends(Updatable):
    __tablename__ = "leagueoflegends"

    @declared_attr
    def region(self):
        return Column(String, nullable=False)

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def user(self):
        return relationship("User", backref="leagueoflegends")

    @declared_attr
    def profile_icon_id(self):
        # 3777
        return Column(Integer, nullable=False)

    @declared_attr
    def summoner_name(self):
        # SteffoRYG
        return Column(String, nullable=False)

    @declared_attr
    def puuid(self):
        # iNW0i7w_cC2kxgNB13UhyGPeyxZChmRqKylZ--bzbZAhFM6EXAImUqeRWmGtK6iKiYbz3bkCV8fMQQ
        return Column(String, nullable=False)

    @declared_attr
    def summoner_level(self):
        # 68
        return Column(Integer, nullable=False)

    @declared_attr
    def summoner_id(self):
        # aEsHyfXA2q8bK-g7GlT4kFK_0uLL3w-jBPyfMAy8kOXTJXo
        return Column(String, nullable=False, primary_key=True)

    @declared_attr
    def account_id(self):
        # -2Ex-VpkkNBN4ceQev8oJsamxY5iGb2liRUqkES5TU_7vtI
        return Column(String, nullable=False)

    @declared_attr
    def rank_soloq_tier(self):
        return Column(Enum(LeagueTier))

    @declared_attr
    def rank_soloq_rank(self):
        return Column(Enum(LeagueRank))

    @declared_attr
    def rank_soloq_points(self):
        return Column(Integer)

    @declared_attr
    def rank_soloq_wins(self):
        return Column(Integer)

    @declared_attr
    def rank_soloq_losses(self):
        return Column(Integer)

    @declared_attr
    def rank_soloq_inactive(self):
        return Column(Boolean)

    @declared_attr
    def rank_soloq_hot_streak(self):
        return Column(Boolean)

    @declared_attr
    def rank_soloq_fresh_blood(self):
        return Column(Boolean)

    @declared_attr
    def rank_soloq_veteran(self):
        return Column(Boolean)

    @declared_attr
    def rank_soloq(self):
        return composite(LeagueLeague,
                         self.rank_soloq_tier,
                         self.rank_soloq_rank,
                         self.rank_soloq_points,
                         self.rank_soloq_wins,
                         self.rank_soloq_losses,
                         self.rank_soloq_inactive,
                         self.rank_soloq_hot_streak,
                         self.rank_soloq_fresh_blood,
                         self.rank_soloq_veteran)

    @declared_attr
    def rank_flexq_tier(self):
        return Column(Enum(LeagueTier))

    @declared_attr
    def rank_flexq_rank(self):
        return Column(Enum(LeagueRank))

    @declared_attr
    def rank_flexq_points(self):
        return Column(Integer)

    @declared_attr
    def rank_flexq_wins(self):
        return Column(Integer)

    @declared_attr
    def rank_flexq_losses(self):
        return Column(Integer)

    @declared_attr
    def rank_flexq_inactive(self):
        return Column(Boolean)

    @declared_attr
    def rank_flexq_hot_streak(self):
        return Column(Boolean)

    @declared_attr
    def rank_flexq_fresh_blood(self):
        return Column(Boolean)

    @declared_attr
    def rank_flexq_veteran(self):
        return Column(Boolean)

    @declared_attr
    def rank_flexq(self):
        return composite(LeagueLeague,
                         self.rank_flexq_tier,
                         self.rank_flexq_rank,
                         self.rank_flexq_points,
                         self.rank_flexq_wins,
                         self.rank_flexq_losses,
                         self.rank_flexq_inactive,
                         self.rank_flexq_hot_streak,
                         self.rank_flexq_fresh_blood,
                         self.rank_flexq_veteran)

    @declared_attr
    def rank_twtrq_tier(self):
        return Column(Enum(LeagueTier))

    @declared_attr
    def rank_twtrq_rank(self):
        return Column(Enum(LeagueRank))

    @declared_attr
    def rank_twtrq_points(self):
        return Column(Integer)

    @declared_attr
    def rank_twtrq_wins(self):
        return Column(Integer)

    @declared_attr
    def rank_twtrq_losses(self):
        return Column(Integer)

    @declared_attr
    def rank_twtrq_inactive(self):
        return Column(Boolean)

    @declared_attr
    def rank_twtrq_hot_streak(self):
        return Column(Boolean)

    @declared_attr
    def rank_twtrq_fresh_blood(self):
        return Column(Boolean)

    @declared_attr
    def rank_twtrq_veteran(self):
        return Column(Boolean)

    @declared_attr
    def rank_twtrq(self):
        return composite(LeagueLeague,
                         self.rank_twtrq_tier,
                         self.rank_twtrq_rank,
                         self.rank_twtrq_points,
                         self.rank_twtrq_wins,
                         self.rank_twtrq_losses,
                         self.rank_twtrq_inactive,
                         self.rank_twtrq_hot_streak,
                         self.rank_twtrq_fresh_blood,
                         self.rank_twtrq_veteran)

    @declared_attr
    def rank_tftq_tier(self):
        return Column(Enum(LeagueTier))

    @declared_attr
    def rank_tftq_rank(self):
        return Column(Enum(LeagueRank))

    @declared_attr
    def rank_tftq_points(self):
        return Column(Integer)

    @declared_attr
    def rank_tftq_wins(self):
        return Column(Integer)

    @declared_attr
    def rank_tftq_losses(self):
        return Column(Integer)

    @declared_attr
    def rank_tftq_inactive(self):
        return Column(Boolean)

    @declared_attr
    def rank_tftq_hot_streak(self):
        return Column(Boolean)

    @declared_attr
    def rank_tftq_fresh_blood(self):
        return Column(Boolean)

    @declared_attr
    def rank_tftq_veteran(self):
        return Column(Boolean)

    @declared_attr
    def rank_tftq(self):
        return composite(LeagueLeague,
                         self.rank_tftq_tier,
                         self.rank_tftq_rank,
                         self.rank_tftq_points,
                         self.rank_tftq_wins,
                         self.rank_tftq_losses,
                         self.rank_tftq_inactive,
                         self.rank_tftq_hot_streak,
                         self.rank_tftq_fresh_blood,
                         self.rank_tftq_veteran)

    @declared_attr
    def mastery_score(self):
        return Column(Integer, nullable=False, default=0)

    def json(self):
        return {
            "region": self.region,
            "profile_icon_id": self.profile_icon_id,
            "summoner_name": self.summoner_name,
            "puuid": self.puuid,
            "summoner_level": self.summoner_level,
            "summoner_id": self.summoner_id,
            "account_id": self.account_id,
            "soloq": self.rank_soloq.json() if self.rank_soloq is not None else None,
            "twtrq": self.rank_twtrq.json() if self.rank_twtrq is not None else None,
            "flexq": self.rank_flexq.json() if self.rank_flexq is not None else None,
            "tftq": self.rank_tftq.json() if self.rank_tftq is not None else None,
        }

    def __repr__(self):
        return f"<{self.__class__.__qualname__} {str(self)}>"

    def __str__(self):
        return f"[c]{self.__tablename__}:{self.summoner_name}[/c]"



from typing import *
import royalnet
import royalnet.commands as rc
import datetime


class PongEvent(rc.Event):
    name = "pong"

    async def run(self, **kwargs) -> dict:
        return {"timestamp": datetime.datetime.now().timestamp()}



from typing import *
import riotwatcher
import logging
import asyncio
import sentry_sdk
import royalnet.commands as rc
import royalnet.utils as ru
import royalnet.serf.telegram as rst
from royalnet.backpack import tables as rbt

from .abstract.linker import LinkerCommand
from ..tables import LeagueOfLegends, FiorygiTransaction
from ..types import LeagueLeague, Updatable

log = logging.getLogger(__name__)


class LeagueoflegendsCommand(LinkerCommand):
    name: str = "leagueoflegends"

    aliases = ["lol", "league"]

    description: str = "Connetti un account di League of Legends a un account Royalnet, o visualizzane le statistiche."

    syntax = "[nomeevocatore]"

    queue_names = {
        "rank_soloq": "Solo/Duo",
        "rank_flexq": "Flex",
        "rank_twtrq": "3v3",
        "rank_tftq": "TFT"
    }

    def __init__(self, interface: rc.CommandInterface):
        super().__init__(interface)
        self._riotwatcher: Optional[riotwatcher.RiotWatcher] = None
        if self.interface.name == "telegram" and self.enabled():
            self._riotwatcher = riotwatcher.RiotWatcher(api_key=self.token())

    def token(self):
        return self.config["leagueoflegends"]["token"]

    def region(self):
        return self.config["leagueoflegends"]["region"]

    def describe(self, obj: LeagueOfLegends) -> str:
        string = f"ℹ️ [b]{obj.summoner_name}[/b]\n" \
                 f"Lv. {obj.summoner_level}\n" \
                 f"Mastery score: {obj.mastery_score}\n" \
                 f"\n"
        if obj.rank_soloq:
            string += f"Solo: {obj.rank_soloq}\n"
        if obj.rank_flexq:
            string += f"Flex: {obj.rank_flexq}\n"
        if obj.rank_twtrq:
            string += f"3v3: {obj.rank_twtrq}\n"
        if obj.rank_tftq:
            string += f"TFT: {obj.rank_tftq}\n"
        return string

    async def get_updatables_of_user(self, session, user: rbt.User) -> List[LeagueOfLegends]:
        return await ru.asyncify(session.query(self.alchemy.get(LeagueOfLegends)).filter_by(user=user).all)

    async def get_updatables(self, session) -> List[LeagueOfLegends]:
        return await ru.asyncify(session.query(self.alchemy.get(LeagueOfLegends)).all)

    async def create(self, session, user: rbt.User, args) -> LeagueOfLegends:
        name = args.joined()

        # Connect a new League of Legends account to Royalnet
        log.debug(f"Searching for: {name}")
        summoner = self._riotwatcher.summoner.by_name(region=self.region(), summoner_name=name)
        # Ensure the account isn't already connected to something else
        leagueoflegends = await ru.asyncify(
            session.query(self.alchemy.get(LeagueOfLegends)).filter_by(summoner_id=summoner["id"]).one_or_none)
        if leagueoflegends:
            raise rc.CommandError(f"L'account {leagueoflegends} è già registrato su Royalnet.")
        # Get rank information
        log.debug(f"Getting leagues data: {name}")
        leagues = self._riotwatcher.league.by_summoner(region=self.region(),
                                                       encrypted_summoner_id=summoner["id"])
        soloq = LeagueLeague()
        flexq = LeagueLeague()
        twtrq = LeagueLeague()
        tftq = LeagueLeague()
        for league in leagues:
            if league["queueType"] == "RANKED_SOLO_5x5":
                soloq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_SR":
                flexq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_TT":
                twtrq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_TFT":
                tftq = LeagueLeague.from_dict(league)
        # Get mastery score
        log.debug(f"Getting mastery data: {name}")
        mastery = self._riotwatcher.champion_mastery.scores_by_summoner(region=self.region(),
                                                                        encrypted_summoner_id=summoner["id"])
        # Create database row
        leagueoflegends = self.alchemy.get(LeagueOfLegends)(
            region=self.region(),
            user=user,
            profile_icon_id=summoner["profileIconId"],
            summoner_name=summoner["name"],
            puuid=summoner["puuid"],
            summoner_level=summoner["summonerLevel"],
            summoner_id=summoner["id"],
            account_id=summoner["accountId"],
            rank_soloq=soloq,
            rank_flexq=flexq,
            rank_twtrq=twtrq,
            rank_tftq=tftq,
            mastery_score=mastery
        )
        session.add(leagueoflegends)
        return leagueoflegends

    async def update(self, session, obj: LeagueOfLegends, change: Callable[[str, Any], Awaitable[None]]):
        log.debug(f"Getting summoner data: {obj}")
        summoner = await ru.asyncify(self._riotwatcher.summoner.by_id, region=self.region(),
                                     encrypted_summoner_id=obj.summoner_id)
        await change("profile_icon_id", summoner["profileIconId"])
        await change("summoner_name", summoner["name"])
        await change("puuid", summoner["puuid"])
        await change("summoner_level", summoner["summonerLevel"])
        await change("summoner_id", summoner["id"])
        await change("account_id", summoner["accountId"])
        log.debug(f"Getting leagues data: {obj}")
        leagues = await ru.asyncify(self._riotwatcher.league.by_summoner, region=self.region(),
                                    encrypted_summoner_id=obj.summoner_id)
        soloq = LeagueLeague()
        flexq = LeagueLeague()
        twtrq = LeagueLeague()
        tftq = LeagueLeague()
        for league in leagues:
            if league["queueType"] == "RANKED_SOLO_5x5":
                soloq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_SR":
                flexq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_TT":
                twtrq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_TFT":
                tftq = LeagueLeague.from_dict(league)
        await change("rank_soloq", soloq)
        await change("rank_flexq", flexq)
        await change("rank_twtrq", twtrq)
        await change("rank_tftq", tftq)
        log.debug(f"Getting mastery data: {obj}")
        mastery = await ru.asyncify(self._riotwatcher.champion_mastery.scores_by_summoner,
                                    region=self.region(),
                                    encrypted_summoner_id=obj.summoner_id)
        await change("mastery_score", mastery)

    async def on_increase(self, session, obj: LeagueOfLegends, attribute: str, old: Any, new: Any) -> None:
        if attribute in self.queue_names.keys():
            await self.notify(f"📈 [b]{obj.user}[/b] è salito a {new} su League of Legends ({self.queue_names[attribute]})! Congratulazioni!")

    async def on_unchanged(self, session, obj: LeagueOfLegends, attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_decrease(self, session, obj: LeagueOfLegends, attribute: str, old: Any, new: Any) -> None:
        if attribute in self.queue_names.keys():
            await self.notify(f"📉 [b]{obj.user}[/b] è sceso a {new} su League of Legends ({self.queue_names[attribute]}).")

    async def on_first(self, session, obj: LeagueOfLegends, attribute: str, old: None, new: Any) -> None:
        if attribute in self.queue_names.keys():
            await self.notify(f"🌟 [b]{obj.user}[/b] si è classificato {new} su League of Legends ({self.queue_names[attribute]}!")

    async def on_reset(self, session, obj: LeagueOfLegends, attribute: str, old: Any, new: None) -> None:
        if attribute in self.queue_names.keys():
            await self.notify(f"⬜️ [b]{obj.user}[/b] non ha più un rank su League of Legends ({self.queue_names[attribute]}).")



import re
import datetime
from sqlalchemy import Column, \
                       Integer, \
                       Text, \
                       Boolean, \
                       DateTime, \
                       ForeignKey, \
                       String
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declared_attr


class Diario:
    __tablename__ = "diario"

    @declared_attr
    def diario_id(self):
        return Column(Integer, primary_key=True)

    @declared_attr
    def creator_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def quoted_account_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def quoted(self):
        return Column(String)

    @declared_attr
    def text(self):
        return Column(Text)

    @declared_attr
    def context(self):
        return Column(Text)

    @declared_attr
    def timestamp(self) -> datetime.datetime:
        return Column(DateTime, nullable=False)

    @declared_attr
    def media_url(self):
        return Column(String)

    @declared_attr
    def spoiler(self):
        return Column(Boolean, default=False)

    @declared_attr
    def creator(self):
        return relationship("User", foreign_keys=self.creator_id, backref="diario_created")

    @declared_attr
    def quoted_account(self):
        return relationship("User", foreign_keys=self.quoted_account_id, backref="diario_quoted")

    def json(self) -> dict:
        return {
            "diario_id": self.diario_id,
            "creator": self.creator.json() if self.creator else None,
            "quoted_account": self.quoted_account.json() if self.quoted_account else None,
            "quoted": self.quoted,
            "text": self.text,
            "context": self.context,
            "timestamp": self.timestamp.isoformat(),
            "media_url": self.media_url,
            "spoiler": self.spoiler
        }

    def __repr__(self):
        return f"<Diario diario_id={self.diario_id}" \
                      f" creator_id={self.creator_id}" \
                      f" quoted_account_id={self.quoted_account_id}" \
                      f" quoted={self.quoted}" \
                      f" text={self.text}" \
                      f" context={self.context}" \
                      f" timestamp={self.timestamp}" \
                      f" media_url={self.media_url}" \
                      f" spoiler={self.spoiler}>"

    def __str__(self):
        text = f"Riga #{self.diario_id} (salvata"
        if self.creator is not None:
            text += f" da {str(self.creator)}"
        text += f" il {self.timestamp.strftime('%Y-%m-%d %H:%M')}):\n"
        if self.media_url is not None:
            text += f"[url={self.media_url}]Media[/url]\n"
        if self.text is not None:
            if self.spoiler:
                hidden = re.sub(r"\w", "█", self.text)
                text += f"\"{hidden}\"\n"
            else:
                text += f"[b]\"{self.text}\"[/b]\n"
        if self.quoted_account is not None:
            text += f" —{str(self.quoted_account)}"
        elif self.quoted is not None:
            text += f" —{self.quoted}"
        else:
            text += f" —Anonimo"
        if self.context:
            text += f", [i]{self.context}[/i]"
        return text



from typing import *
import random
import royalnet.commands as rc

from .play import PlayCommand


class ElevatormusicCommand(PlayCommand):
    name: str = "elevatormusic"

    aliases = ["elevator", "em"]

    description: str = "Aggiungi un po' di musica da ascensore alla chat vocale."

    syntax = "[indice]"

    _pool = [
        "https://www.youtube.com/watch?v=_tAcIGhh5Yo",
        "https://www.youtube.com/watch?v=xy_NKN75Jhw",
        "https://www.youtube.com/watch?v=PLRrL9OsAF8",
        "https://www.youtube.com/watch?v=0TmoYBcLul8",
        "https://www.youtube.com/watch?v=9v9-Nw4nAZg",
        "https://www.youtube.com/watch?v=VBlFHuCzPgY",
        "https://www.youtube.com/watch?v=mD3v1B_aXw0",
        "https://www.youtube.com/watch?v=RCSbmSmyAcY",
        "https://www.youtube.com/watch?v=kbdtBLD8Lbg",
        "https://www.youtube.com/watch?v=JjT0p2z4hGg",
        "https://www.youtube.com/watch?v=cc_KpOufpgM",
        "https://www.youtube.com/watch?v=dlNEUYBt7Ls",
        "https://www.youtube.com/watch?v=YVWdQGuE0-E",
        "https://www.youtube.com/watch?v=1nVAg8VujGA",
        "https://www.youtube.com/watch?v=G4mshu6BUio",
        "https://www.youtube.com/watch?v=Y6_Aij3n8hk",
        "https://www.youtube.com/watch?v=SbIaYZEUF1M",
        "https://www.youtube.com/watch?v=Nf8FCLT8S6A",
    ]

    async def get_urls(self, args):
        index = args.optional(0)
        if index is not None:
            try:
                return [self._pool[int(index)]]
            except ValueError:
                raise rc.InvalidInputError("L'indice deve essere un numero.\n"
                                           f"Gli indici delle canzoni iniziano a [c]0[/c] e finiscono a"
                                           f" [c]{len(self._pool) - 1}[/c].")
            except IndexError:
                raise rc.InvalidInputError(f"Indice non valido.\n"
                                           f"Gli indici delle canzoni iniziano a [c]0[/c] e finiscono a"
                                           f" [c]{len(self._pool) - 1}[/c].")
        else:
            return [random.sample(self._pool, 1)[0]]



import datetime
import asyncio
from typing import *
import royalnet
import royalnet.commands as rc


class PingCommand(rc.Command):
    name: str = "ping"

    description: str = "Display the status of the Herald network."

    syntax: str = ""

    _targets = ["telegram", "discord", "matrix", "constellation"]

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        await data.reply("📶 Ping...")

        tasks = {}

        start = datetime.datetime.now()
        for target in self._targets:
            tasks[target] = self.loop.create_task(self.interface.call_herald_event(target, "pong"))

        await asyncio.sleep(10)

        lines = ["📶 [b]Pong![/b]", ""]

        for name, task in tasks.items():
            try:
                d = task.result()
            except (asyncio.CancelledError, asyncio.InvalidStateError):
                lines.append(f"🔴 [c]{name}[/c]")
            else:
                end = datetime.datetime.fromtimestamp(d["timestamp"])
                delta = end - start

                lines.append(f"🔵 [c]{name}[/c] ({delta.microseconds // 1000} ms)")

        await data.reply("\n".join(lines))



from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr


class Treasure:
    __tablename__ = "treasures"

    @declared_attr
    def code(self):
        return Column(String, primary_key=True)

    @declared_attr
    def redeemed_by_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def redeemed_by(self):
        return relationship("User")

    @declared_attr
    def value(self):
        return Column(Integer, nullable=False)



from typing import *
import royalnet
import royalnet.commands as rc


class HelpCommand(rc.Command):
    name: str = "help"

    description: str = "Visualizza informazioni su un comando."

    syntax: str = "{comando}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        name: str = args[0].lstrip(self.interface.prefix)

        try:
            command: rc.Command = self.serf.commands[f"{self.interface.prefix}{name}"]
        except KeyError:
            raise rc.InvalidInputError("Il comando richiesto non esiste.")

        message = [
            f"[c]{self.interface.prefix}{command.name} {command.syntax}[/c]",
            "",
            f"{command.description}"
        ]

        await data.reply("\n".join(message))



from typing import *
import discord
import royalnet.commands as rc


class PauseCommand(rc.Command):
    name: str = "pause"

    aliases = ["resume"]

    description: str = "Metti in pausa o riprendi la riproduzione di un file."

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild: discord.Guild = message.guild
            guild_id: Optional[int] = guild.id
        else:
            guild_id = None
        response: Dict[str, Any] = await self.interface.call_herald_event("discord", "discord_pause",
                                                                          guild_id=guild_id)

        if response["action"] == "paused":
            await data.reply("⏸ Riproduzione messa in pausa.")

        elif response["action"] == "resumed":
            await data.reply("▶️ Riproduzione ripresa!")



import re
import royalnet.utils as ru
import royalnet.constellation.api as rca


url_validation = re.compile(r'^(?:http|ftp)s?://'
                            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
                            r'localhost|'
                            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
                            r'(?::\d+)?'
                            r'(?:/?|[/?]\S+)$', re.IGNORECASE)


class ApiUserAvatarStar(rca.ApiStar):
    path = "/api/user/avatar/v2"

    parameters = {
        "put": {
            "avatar_url": "The url that the user wants to set as avatar."
        }
    }

    auth = {
        "put": True,
    }

    tags = ["user"]

    @rca.magic
    async def put(self, data: rca.ApiData) -> ru.JSON:
        """Set the avatar of current user."""
        avatar_url = data["avatar_url"]
        user = await data.user()
        if not re.match(url_validation, avatar_url):
            raise rca.InvalidParameterError("avatar_url is not a valid url.")
        user.avatar_url = avatar_url
        await data.session_commit()
        return user.json()



import enum


class DotaStars(enum.Enum):
    I = 1
    II = 2
    III = 3
    IV = 4
    V = 5
    VI = 6
    VII = 7

    def __str__(self):
        return self.name.upper()

    def __repr__(self):
        return f"{self.__class__.__qualname__}.{self.name}"

    def __gt__(self, other):
        return self.value > other.value



from typing import *
import discord
import asyncio as aio
import royalnet.commands as rc
import royalnet.backpack.tables as rbt


class LazyplayCommand(rc.Command):
    name: str = "lazyplay"

    aliases = ["lp"]

    description: str = "Aggiunge un url alla coda della chat vocale, ma lo scarica solo quando sta per essere" \
                       " riprodotto."

    syntax = "{url}"

    async def get_urls(self, args: rc.CommandArgs):
        url = args.joined(require_at_least=1)
        if not (url.startswith("http://") or url.startswith("https://")):
            raise rc.InvalidInputError(f"L'URL specificato non inizia con il nome di un protocollo supportato"
                                    f" ([c]http://[/c] o [c]https://[/c]).")
        return [url]

    def get_embed_color(self) -> Optional[int]:
        return None

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild: discord.Guild = message.guild
            if guild is None:
                guild_id = None
            else:
                guild_id: Optional[int] = guild.id
        else:
            guild_id = None

        user: rbt.User = await data.get_author()
        user_str = None

        if user is not None:
            try:
                user_discord: rbt.Discord = user.discord[0]
            except (AttributeError, IndexError):
                user_str = str(user)
            else:
                user_str = str(f"<@{user_discord.discord_id}>")

        urls = await self.get_urls(args)

        play_task: aio.Task = self.loop.create_task(
            self.interface.call_herald_event("discord", "discord_lazy_play",
                                             urls=urls,
                                             guild_id=guild_id,
                                             user=user_str,
                                             force_color=self.get_embed_color())
        )

        await data.reply("⌛ Attendi un attimo...")

        await play_task



from typing import *

import asyncio
import logging
import aiohttp

from royalnet.backpack import tables as rbt
from royalnet.commands import *
from royalnet.utils import *
from sqlalchemy import or_, and_

from .abstract.linker import LinkerCommand
from ..tables import Steam, Brawlhalla, BrawlhallaDuo
from ..types import BrawlhallaRank, BrawlhallaMetal, BrawlhallaTier, Updatable

log = logging.getLogger(__name__)


class BrawlhallaCommand(LinkerCommand):
    name: str = "brawlhalla"

    aliases = ["bh", "bruhalla", "bruhlalla"]

    description: str = "Visualizza le tue statistiche di Brawlhalla."

    syntax: str = ""

    def token(self):
        return self.config['brawlhalla']['token']

    async def get_updatables_of_user(self, session, user: rbt.User) -> List[Brawlhalla]:
        return user.steam

    async def get_updatables(self, session) -> List[Brawlhalla]:
        return await asyncify(session.query(self.alchemy.get(Steam)).all)

    async def create(self, session, user: rbt.User, args) -> Updatable:
        raise InvalidInputError("Brawlhalla accounts are automatically linked from Steam.")

    async def update(self, session, obj, change: Callable[[str, Any], Awaitable[None]]):
        BrawlhallaT = self.alchemy.get(Brawlhalla)
        DuoT = self.alchemy.get(BrawlhallaDuo)
        log.info(f"Updating: {obj}")
        async with aiohttp.ClientSession() as hcs:
            bh: Brawlhalla = obj.brawlhalla
            if bh is None:
                log.debug(f"Checking if player has an account...")
                async with hcs.get(f"https://api.brawlhalla.com/search?steamid={obj.steamid.as_64}&api_key={self.token()}") as response:
                    if response.status != 200:
                        raise ExternalError(f"Brawlhalla API /search returned {response.status}!")
                    j = await response.json()
                    if j == {} or j == []:
                        log.debug("No account found.")
                        return
                    bh = BrawlhallaT(
                        steam=obj,
                        brawlhalla_id=j["brawlhalla_id"],
                        name=j["name"]
                    )
                    session.add(bh)
                    session.flush()

            async with hcs.get(f"https://api.brawlhalla.com/player/{bh.brawlhalla_id}/ranked?api_key={self.token()}") as response:
                if response.status != 200:
                    raise ExternalError(f"Brawlhalla API /ranked returned {response.status}!")
                j = await response.json()
                if j == {} or j == []:
                    log.debug("No ranked info found.")
                else:
                    await self._change(session=session, obj=bh, attribute="rating_1v1", new=j["rating"])
                    metal_name, tier_name = j["tier"].split(" ", 1)
                    metal = BrawlhallaMetal[metal_name.upper()]
                    tier = BrawlhallaTier(int(tier_name))
                    rank = BrawlhallaRank(metal=metal, tier=tier)
                    await self._change(session=session, obj=bh, attribute="rank_1v1", new=rank)

                    for jduo in j.get("2v2", []):
                        bhduo: Optional[BrawlhallaDuo] = await asyncify(
                            session.query(DuoT)
                                .filter(
                                    or_(
                                        and_(
                                            DuoT.id_one == jduo["brawlhalla_id_one"],
                                            DuoT.id_two == jduo["brawlhalla_id_two"]
                                        ),
                                        and_(
                                            DuoT.id_one == jduo["brawlhalla_id_two"],
                                            DuoT.id_two == jduo["brawlhalla_id_one"]
                                        )
                                    )
                                )
                                .one_or_none
                        )
                        if bhduo is None:
                            if bh.brawlhalla_id == jduo["brawlhalla_id_one"]:
                                otherbh: Optional[Brawlhalla] = await asyncify(
                                    session.query(BrawlhallaT).get, jduo["brawlhalla_id_two"]
                                )
                            else:
                                otherbh: Optional[Brawlhalla] = await asyncify(
                                    session.query(BrawlhallaT).get, jduo["brawlhalla_id_one"]
                                )
                            if otherbh is None:
                                continue
                            bhduo = DuoT(
                                one=bh,
                                two=otherbh,
                            )

                            session.add(bhduo)
                        await self._change(session=session, obj=bhduo, attribute="rating_2v2", new=jduo["rating"])
                        metal_name, tier_name = jduo["tier"].split(" ", 1)
                        metal = BrawlhallaMetal[metal_name.upper()]
                        tier = BrawlhallaTier(int(tier_name))
                        rank = BrawlhallaRank(metal=metal, tier=tier)
                        await self._change(session=session, obj=bhduo, attribute="rank_2v2", new=rank)

    async def on_increase(self, session, obj: Union[Brawlhalla, BrawlhallaDuo], attribute: str, old: Any, new: Any) -> None:
        if attribute == "rank_1v1":
            await self.notify(f"📈 [b]{obj.steam.user}[/b] è salito a [b]{new}[/b] ({obj.rating_1v1} MMR) in 1v1 su Brawlhalla! Congratulazioni!")
        elif attribute == "rank_2v2":
            await self.notify(f"📈 [b]{obj.one.steam.user}[/b] e [b]{obj.two.steam.user}[/b] sono saliti a [b]{new}[/b] ({obj.rating_2v2} MMR) in 2v2 su Brawlhalla! Congratulazioni!")

    async def on_unchanged(self, session, obj: Union[Brawlhalla, BrawlhallaDuo], attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_decrease(self, session, obj: Union[Brawlhalla, BrawlhallaDuo], attribute: str, old: Any, new: Any) -> None:
        if attribute == "rank_1v1":
            await self.notify(f"📉 [b]{obj.steam.user}[/b] è sceso a [b]{new}[/b] ({obj.rating_1v1} MMR) in 1v1 su Brawlhalla.")
        elif attribute == "rank_2v2":
            await self.notify(f"📉 [b]{obj.one.steam.user}[/b] e [b]{obj.two.steam.user}[/b] sono scesi a [b]{new}[/b] ({obj.rating_2v2} MMR) in 2v2 su Brawlhalla.")

    async def on_first(self, session, obj: Union[Brawlhalla, BrawlhallaDuo], attribute: str, old: None, new: Any) -> None:
        if attribute == "rank_1v1":
            await self.notify(f"🌟 [b]{obj.steam.user}[/b] si è classificato a [b]{new}[/b] ({obj.rating_1v1} MMR) in 1v1 su Brawlhalla!")
        elif attribute == "rank_2v2":
            await self.notify(f"🌟 [b]{obj.one.steam.user}[/b] e [b]{obj.two.steam.user}[/b] si sono classificati a [b]{new}[/b] ({obj.rating_2v2} MMR) in 2v2 su Brawlhalla!")

    async def on_reset(self, session, obj: Union[Brawlhalla, BrawlhallaDuo], attribute: str, old: Any, new: None) -> None:
        if attribute == "rank_1v1":
            await self.notify(f"⬜️ [b]{obj.steam.user}[/b] non ha più un rank su Brawlhalla.")
        elif attribute == "rank_2v2":
            await self.notify(f"⬜️ [b]{obj.one.steam.user}[/b] e [b]{obj.two.steam.user}[/b] non hanno più un rank su Brawlhalla.")

    def describe(self, obj: Steam) -> str:
        bh = obj.brawlhalla

        string = [f"ℹ️ [b]{bh.name}[/b]", ""]

        if bh.rank_1v1:
            string.append("👤 [b]1v1[/b]")
            string.append(f"[b]{bh.rank_1v1}[/b] ({bh.rating_1v1} MMR)")
            string.append("")

        if len(bh.duos) != 0:
            string.append(f"👥 [b]2v2[/b]")

        for duo in bh.duos:
            other = duo.other(bh)
            string.append(f"Con [b]{other.steam.user}[/b]: [b]{duo.rank_2v2}[/b] ({duo.rating_2v2} MMR)")

        if len(bh.duos) != 0:
            string.append("")

        return "\n".join(string)



from sqlalchemy import Column, \
                       Integer, \
                       ForeignKey
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr


class TriviaScore:
    __tablename__ = "triviascores"

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("users.uid"), primary_key=True)

    @declared_attr
    def user(self):
        return relationship("User", backref=backref("trivia_score", uselist=False))

    @declared_attr
    def correct_answers(self):
        return Column(Integer, nullable=False, default=0)

    @declared_attr
    def wrong_answers(self):
        return Column(Integer, nullable=False, default=0)

    @property
    def total_answers(self):
        return self.correct_answers + self.wrong_answers

    @property
    def offset(self):
        return self.correct_answers - self.wrong_answers

    @property
    def correct_rate(self):
        if self.total_answers == 0:
            return 0.0
        return self.correct_answers / self.total_answers

    @property
    def score(self) -> float:
        return (self.correct_answers + self.correct_answers * self.correct_rate) * 10

    def json(self):
        return {
            "correct": self.correct_answers,
            "wrong": self.wrong_answers,
            "total": self.total_answers,
            "rate": self.correct_rate,
            "score": self.score
        }

    def __repr__(self):
        return f"<TriviaScore of {self.user}: ({self.correct_answers}|{self.wrong_answers})>"



import royalnet.utils as ru
import royalnet.backpack.tables as rbt
import royalnet.constellation.api as rca
from ..tables import Bio


class ApiBioSetStar(rca.ApiStar):
    path = "/api/bio/v2"

    parameters = {
        "get": {
            "uid": "The id of the user to get the bio of."
        },
        "put": {
            "contents": "The contents of the bio."
        }
    }

    auth = {
        "get": False,
        "put": True,
    }

    tags = ["bio"]

    @rca.magic
    async def get(self, data: rca.ApiData) -> ru.JSON:
        """Get the bio of a specific user."""
        user = await rbt.User.find(self.alchemy, data.session, data.int("uid"))
        return user.bio.json() if user.bio else None

    @rca.magic
    async def put(self, data: rca.ApiData) -> ru.JSON:
        """Set the bio of current user."""
        contents = data["contents"]
        BioT = self.alchemy.get(Bio)
        user = await data.user()
        bio = user.bio
        if bio is None:
            bio = BioT(user=user, contents=contents)
            data.session.add(bio)
        else:
            bio.contents = contents
        await data.session_commit()
        return bio.json()
